@@ -1,12 +1,12 @@
 ;;; fontaine.el --- Set font configurations using presets -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022  Free Software Foundation, Inc.
+;; Copyright (C) 2022-2023  Free Software Foundation, Inc.
 
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
 ;; Maintainer: Fontaine Development <~protesilaos/fontaine@lists.sr.ht>
 ;; URL: https://git.sr.ht/~protesilaos/fontaine
 ;; Mailing-List: https://lists.sr.ht/~protesilaos/fontaine
-;; Version: 0.4.1
+;; Version: 1.0.0
 ;; Package-Requires: ((emacs "27.1"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -108,10 +108,11 @@
 ;; not stored in their `.emacs.d', but in a standard directory instead:
 ;; <https://github.com/emacscollective/no-littering>.
 ;;
-;; As for the name of this package, it is the French word for "fountain"
-;; which, in turn, is what the font or source is.  However, I will not
-;; blame you if you can only interpret it as a descriptive acronym: FONTs
-;; Are Irrelevant in Non-graphical Emacs (because that is actually true).
+;; As for the name of this package, it is the French word for
+;; "fountain" which, in turn, is what the font or source is.  However,
+;; I will not blame you if you can only interpret it as a descriptive
+;; backronym: Fonts, Ornaments, and Neat Typography Are Irrelevant in
+;; Non-graphical Emacs.
 
 ;;; Code:
 
@@ -122,19 +123,19 @@
   :group 'font)
 
 (defconst fontaine--weights-widget
-  '(choice
-    (const :tag "Normal" normal)
-    (const :tag "Regular (same as normal)" regular)
-    (const :tag "Thin" thin)
-    (const :tag "Ultra-light" ultralight)
-    (const :tag "Extra-light" extralight)
-    (const :tag "Light" light)
-    (const :tag "Semi-light" semilight)
-    (const :tag "Medium" medium)
-    (const :tag "Semi-bold" semibold)
-    (const :tag "Bold" bold)
-    (const :tag "Extra-bold" extrabold)
-    (const :tag "Ultra-bold" ultrabold))
+  '(choice :tag "Font weight (must be supported by the typeface)"
+           (const :tag "Normal" normal)
+           (const :tag "Regular (same as normal)" regular)
+           (const :tag "Thin" thin)
+           (const :tag "Ultra-light" ultralight)
+           (const :tag "Extra-light" extralight)
+           (const :tag "Light" light)
+           (const :tag "Semi-light" semilight)
+           (const :tag "Medium" medium)
+           (const :tag "Semi-bold" semibold)
+           (const :tag "Bold" bold)
+           (const :tag "Extra-bold" extrabold)
+           (const :tag "Ultra-bold" ultrabold))
   "Widget with font weights for `fontaine-presets'.")
 
 (defcustom fontaine-presets
@@ -225,6 +226,18 @@ The properties in detail:
 - The `:line-spacing' specifies the value of the `line-spacing'
   variable.
 
+- The `:inherit' contains the name of another named preset.  This
+  tells the relevant Fontaine functions to get the properties of
+  that preset and blend them with those of the current one.  The
+  properties of the current preset take precedence over those of
+  the inherited one, thus overriding them.  In practice, this is
+  a way to have something like an extra-large preset copy the
+  large preset and then only modify its individual properties.
+  Remember that all named presets fall back to the preset whose
+  name is t: the `:inherit' is not a substitute for that generic
+  fallback but rather an extra method of specifying font
+  configuration presets.
+
 Use the desired preset with the command `fontaine-set-preset'.
 
 For detailed configuration: Info node `(fontaine) Shared and
@@ -281,12 +294,25 @@ Caveats or further notes:
                     (const reverse-italic)
                     (const reverse-oblique)))
 
-                  ((const :tag "Line spacing" :line-spacing) ,(get 'line-spacing 'custom-type))))
+                  ((const :tag "Line spacing" :line-spacing) ,(get 'line-spacing 'custom-type))
+                  ;; FIXME 2023-01-19: Adding the (choice
+                  ;; ,@(fontaine--inheritable-presets-widget)) instead
+                  ;; of `symbol' does not have the desired effect
+                  ;; because it does not re-read `fontaine-presets'.
+                  ((const :tag "Inherit another preset" :inherit) symbol)))
           :key-type symbol)
-  :package-version '(fontaine . "0.4.0")
+  :package-version '(fontaine . "0.5.0")
   :group 'fontaine
   :link '(info-link "(fontaine) Shared and implicit fallback values for presets"))
 
+;; ;; See FIXME above in `fontaine-presets' :type.
+;; ;;
+;; (defun fontaine--inheritable-presets-widget ()
+;;   "Return widget with choice among named presets."
+;;   (mapcar (lambda (s)
+;;             (list 'const s))
+;;           (delq t (mapcar #'car fontaine-presets))))
+
 (defcustom fontaine-latest-state-file
   (locate-user-emacs-file "fontaine-latest-state.eld")
   "File to save the latest value of `fontaine-set-preset'.
@@ -374,14 +400,32 @@ combine the other two lists."
 
 ;;;; Apply preset configurations
 
+(defun fontaine--preset-p (preset)
+  "Return non-nil if PRESET is one of the named `fontaine-presets'."
+  (let ((presets (delq t (mapcar #'car fontaine-presets))))
+    (memq preset presets)))
+
+(defun fontaine--get-inherit-name (preset)
+  "Get the `:inherit' value of PRESET."
+  (when-let* ((inherit (plist-get (alist-get preset fontaine-presets) :inherit))
+              (fontaine--preset-p inherit))
+    inherit))
+
+(defun fontaine--get-preset-properties (preset)
+  "Return list of properties for PRESET in `fontaine-presets'."
+  (let ((presets fontaine-presets))
+    (append (alist-get preset presets)
+            (when-let ((inherit (fontaine--get-inherit-name preset)))
+              (alist-get inherit presets))
+            (alist-get t presets))))
+
 (defmacro fontaine--apply-preset (fn doc args)
   "Produce function to apply preset.
 FN is the symbol of the function, DOC is its documentation, and
 ARGS are its routines."
   `(defun ,fn (preset &optional frame)
      ,doc
-     (if-let ((properties (append (alist-get preset fontaine-presets)
-                                  (alist-get t fontaine-presets))))
+     (if-let ((properties (fontaine--get-preset-properties preset)))
          ,args
        ;; FIXME 2022-09-07: Because we `append' the t of
        ;; `fontaine-presets' this error is only relevant when the list
@@ -438,6 +482,7 @@ ARGS are its routines."
   'bold
   (or (plist-get properties :bold-family) 'unspecified)
   (or (plist-get properties :bold-weight) 'bold)
+  'unspecified
   frame))
 
 (fontaine--apply-preset
@@ -658,7 +703,7 @@ face."
   "Set `bold' attributes, optionally for FRAME."
   (let ((weight (intern (completing-read "Select weight for `bold': "
                                          fontaine--font-weights nil t))))
-    (fontaine--set-face-attributes 'bold 'unspecified weight frame)))
+    (fontaine--set-face-attributes 'bold 'unspecified weight 'unspecified frame)))
 
 (defun fontaine--set-italic (&optional frame)
   "Set `italic' attributes, optionally for FRAME."
